@@ -1,7 +1,7 @@
 #ifndef _NFT_ARP_H_
 #define _NFT_ARP_H_
 
-extern char *opcodes[];
+extern char *arp_opcodes[];
 #define NUMOPCODES 9
 
 #endif
